﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using kasrsdk;
using System.Threading;

namespace test_kasrsdk
{
    class MyCallBack : KAsrCallBack
    {
        public static readonly object _locker = new object();
        public void onWarning(ASR_SDK_WARNING warningType, string content)
        {
            Console.WriteLine("onWarning");
        }
        public void onError(ASR_SDK_ERROR errorType, string content)
        {
            Console.WriteLine("onError");
        }
        public void onRelease(Int32 result)
        {
            Console.WriteLine("onRelease");
        }

        public void onStart(Int32 result, string sessionId)
        {
            Console.WriteLine("onStart " + result.ToString());
            if(result == 0) // suc
            {
                lock (_locker)
                {
                    Monitor.Pulse(_locker);
                }
                   
            }
        }

        public  void onRecogResult(KAsrRecogResult result)
        {
            Console.WriteLine(result.fin.ToString() + " " + result.text);
        }

        public void onStop(Int32 result)
        {
            Console.WriteLine("onStop " + result.ToString());
        }

    }
 

    class Token
    {
        const int MAX_RANDOM_NUMBER = 99999999;

        public Token(string appId , string appSecret)
        {
            this.m_appId = appId;
            this.m_appSecret = appSecret;
        }

     public string   getToken(int expire = 7200, string userId = "") 
        {
            var random = new Random();
            string salt = random.Next(0, MAX_RANDOM_NUMBER).ToString();

            Int64 epoch = (DateTime.Now.ToUniversalTime().Ticks - 621355968000000000) / 10000000;
            var expire_tick = epoch + expire;
            string signature = m_appId + userId + salt + expire_tick.ToString();
            var encoding = new System.Text.ASCIIEncoding();
            byte[] keyByte = encoding.GetBytes(m_appSecret);
            byte[] messageBytes = encoding.GetBytes(signature);
            using (var hmacsha256 = new HMACSHA256(keyByte))
            {
                byte[] hashmessage = hmacsha256.ComputeHash(messageBytes);
                
               return m_version + ":" + m_appId + ":" + salt + ":" + expire_tick.ToString() + ":" + Convert.ToBase64String(hashmessage);
            }

               
    }

        string m_version = "1";
        string m_appId;
        string m_appSecret;
    }

    class Program
    {
        static void Main(string[] args)
        {

            if (args.Length < 3)
            {
                Console.WriteLine("test_kasrsdk pcmpath appid appsecret");
                return;
            }
            
            if (!File.Exists(args[0]))
            {
                Console.WriteLine("not exist " + args[0]);
                return;
            }
                         
            Token token = new Token(args[1], args[2]); 

            KAsrSdkLogParam logparam ;
            logparam.callback = null;
            logparam.enableLog = true;
            logparam.logPath = null;
            KAsrSdkInitParam initparam;
            var ret = KAsrClient.init(logparam, initparam);
            Console.WriteLine("init " + ret.ToString());
            KAsrClient client = KAsrClient.createAsrInstance();
            MyCallBack callback = new MyCallBack();
            ret = client.init(callback);
            KAsrAuthParam authparam;
            authparam.serverAddr = "ws://139.196.210.175:7961";
            authparam.token = token.getToken();
            ret = client.start(authparam);
            Console.WriteLine("start " + ret.ToString());

            lock(MyCallBack._locker)
            {
                Monitor.Wait(MyCallBack._locker);
            }

            BinaryReader br = new BinaryReader(File.Open(args[0], FileMode.Open));
            int buff_len = 2 * 40 * 16;

            byte[] buffer;
            do
            {
                buffer = br.ReadBytes(buff_len);
                KAsrAudioFrameData data;
                data.data = buffer;
                data.channelCount = 1;
                data.bitWidth = 16;
                data.sampleRate = 16000;
                client.feed(data);
                Thread.Sleep(40);
            } while (buffer.Length == buff_len);

           
            client.stop();
            Thread.Sleep(10);
            KAsrClient.destoryAsrInstance(client);
            KAsrClient.release();


            Console.ReadLine();

        }
    }
}
